@@ -75,15 +75,17 @@ class Gateway:
             if msg.type == aiohttp.WSMsgType.BINARY:
                 self.buffer.extend(msg.data)
                 raw = inflator.decompress(self.buffer).decode("utf-8")
+                if len(msg.data) < 4 or data[-4:] != ZLIB_SUFFIX:
+                    raise
                 self.buffer = bytearray()  # clean buffer
                 data = json.loads(raw)
-                self._seq = data["s"]
                 _log.debug("> %s", data)
 
+                if data["s"] is not None:
+                        self._seq = data["s"]
+
                 if data["op"] == 0:
                     self.dis.dispatch(data["t"], data["d"])
-                    if data["s"] is not None:
-                        self._seq = data["s"]
                     if (
                         data["t"] == "READY"
                     ):  # only fire up getting the session_id once ready.
@@ -161,10 +163,19 @@ class Gateway:
         await self.connect()
 
     async def heartbeat(self, interval: float):
+        if self._seq is None:
+            await self.close(1008)
+            await self.connect()
+            return
         await self.send({"op": 1, "d": self._seq})
         await asyncio.sleep(interval)
         self.state.loop.create_task(self.heartbeat(interval))
 
+    async def close(self, code: int = 1000):
+        if self.ws:
+            await self.ws.close(code=code)
+        self.buffer.clear()
+
     async def hello(self, data):
         interval = data["d"]["heartbeat_interval"] / 1000
         init = interval * random()
